package com.mortals.xhx.modules.implementlist.listener;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.mortals.xhx.common.utils.SnowFlakeUtil;
import com.mortals.xhx.modules.implementlist.dao.ImplementlistDao;
import com.mortals.xhx.modules.implementlist.model.ImplementlistEntity;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;

/**
 * @author liaozimu
 * @Date: 2021-11-2021/11/2-17:23
 * @Description:
 */
@Slf4j
public class ImplementImportListener extends AnalysisEventListener<ImplementlistEntity> {

    private ImplementlistDao implementlistDao;

    /**
     * 每隔5条存储数据库，实际使用中可以3000条，然后清理list ，方便内存回收
     */
    private static final int BATCH_COUNT = 3000;

    /**
     * 这个集合用于接收 读取Excel文件得到的数据
     */
    List<ImplementlistEntity> entityList = new ArrayList<ImplementlistEntity>();


    public ImplementImportListener(ImplementlistDao implementlistDao) {
        this.implementlistDao = implementlistDao;
    }

    @SneakyThrows
    @Override
    public void invoke(ImplementlistEntity implementlistEntity, AnalysisContext analysisContext) {
        implementlistEntity.setId(SnowFlakeUtil.get().nextId());
        entityList.add(implementlistEntity);
        //达到BATCH_COUNT了，需要去存储一次数据库，防止数据几万条数据在内存，容易OOM
        if (entityList.size() >= BATCH_COUNT) {
            implementlistDao.insertBatch(entityList);
            //存储完成清理list
            entityList.clear();
        }
        log.info("=======Excel文件<<>>导入成功===========");
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        // 这里也要保存数据，确保最后遗留的数据也存储到数据库
        implementlistDao.insertBatch(entityList);
        log.info("=======Excel文件<<>>导入成功===========");
    }
}
